/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-4 - password.c - Returning data to a widget 
 *  
 *  HDR2: Chapter 8, page 183
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/****** Include required header files *****************************/

#include <X11/Intrinsic.h>       /* The Intrinsics                */
#include <Xm/Xm.h>               /* Resource symbols              */
#include <Xm/Form.h>             /* Motif XmForm widget           */
#include <Xm/TextF.h>            /* Motif XmTextField widget      */

/****** Declare global variables **********************************/

char password[20];               /* password input buffer          */
char current_pos;                /* current input position         */
void EchoInput();                /* Verify alpha input             */
void CheckPassword();            /* Verify password is OK          */
void ClearInput();               /* Clear field                    */

/****** Start of main() ********************************************/

main(argc,argv)
int  argc;
char **argv;
{
  XtAppContext  context;                      /* Appl context      */
  Widget     toplevel, form, textf;           /* Widget tree       */

/******* Create widget tree, bind callbacks  ***********************/

  toplevel = XtAppInitialize(&context, "Password", NULL, 0, 
      &argc, argv, NULL, NULL, 0);

  form = XtCreateManagedWidget( "form", xmFormWidgetClass, 
      toplevel, NULL, 0);

  textf = XtCreateManagedWidget( "textf", xmTextFieldWidgetClass, 
      form, NULL, 0);

  password[0] = NULL;                   /* Initialize input buffer */
  current_pos = 0; 

  XtAddCallback( textf, XmNfocusCallback, ClearInput, NULL);
  XtAddCallback( textf, XmNmodifyVerifyCallback, EchoInput, NULL);
  XtAddCallback( textf, XmNactivateCallback, CheckPassword, NULL);

  XtRealizeWidget(toplevel);
  XtAppMainLoop(context);
}

/****** Callback Routines ******************************************/

void ClearInput( w, client_data, call_data )
Widget   w;
XtPointer client_data;                              /* unused here */
XtPointer call_data;                                /* unused here */
{

  XtVaSetValues( w, XmNvalue, "", NULL);            /* Clear text  */

  password[0] = NULL;                               /* Clear buffer*/
  current_pos = 0;

}

/*******************************************************************/

void EchoInput( w, client_data, call_data )
Widget   w;
XtPointer client_data;                              /* unused here */
XtPointer call_data;               /* XmTextVerifyCallbackStruct*  */
{
  int i;
  /* cast call_data */
  XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct*) call_data;

  /* pre-scan input  */
  for ( i=0; i<cbs->text->length; i++){ 
    if(cbs->text->ptr[i] < 'A' || cbs->text->ptr[i] > 'z'){
      cbs->doit = False;
      return;                                   /* reject bad char */
     }
  }
  
  for ( i=0; i<cbs->text->length; i++){
    password[current_pos] = cbs->text->ptr[i];  /* save real char  */
    cbs->text->ptr[i] = 'X';                    /* Substitute "X"  */
    current_pos++;     
    password[current_pos] = NULL;               /* NULL-terminate  */
  }
}

/*******************************************************************/

void CheckPassword( w, client_data, call_data )
Widget   w;
XtPointer client_data;                            /* unused here   */
XtPointer call_data;                              /* unused here   */
{
/* Check input     */
  if ( !strcmp(password, "toolkits") ) {
    XtRemoveCallback(w, XmNmodifyVerifyCallback, EchoInput, NULL);
    XtVaSetValues( w, XmNvalue, "Access Granted!", NULL);
  }
  else {
    XtVaSetValues( w, XmNvalue, "", NULL);        /* Clear text    */
    XBell(XtDisplay(w), 50);                      /* Beep at user  */
    password[0] = NULL;                           /* Clear buffer  */
    current_pos = 0;
  }
}
